from ..WikipediaListGuide import WikipediaListGuide
from .. WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

SPECIES = 'species'


def get_domesticated_animals(wikipedia=None, echo=0):
	return {'domesticated_animals': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'domesticated'],
		data_type=SPECIES,
		urls='https://en.wikipedia.org/wiki/List_of_domesticated_animals',
		representation='table',
		columns={SPECIES: ['animal', 'animal_name', 'species_and_subspecies']}
	)}


def get_pests(wikipedia=None, echo=0):
	return {'common_household_pests': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'pest'],
		data_type=SPECIES,
		urls='https://en.wikipedia.org/wiki/List_of_common_household_pests',
		representation='table',
		columns={SPECIES: ['animal', 'animal_name', 'species_and_subspecies']}
	)}


def get_herbivores(wikipedia=None, echo=0):
	return {'herbivorous_animals': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'herbivore'],
		data_type=SPECIES,
		urls='https://en.wikipedia.org/wiki/List_of_herbivorous_animals',
		representation='table',
		columns={SPECIES: ['animal', 'animal_name', 'species_and_subspecies']}
	)}


def get_animals(wikipedia=None, echo=0):
	return {'animal_names': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal'],
		data_type=SPECIES,
		urls=[
			'https://en.wikipedia.org/wiki/List_of_animal_names',
			'https://en.wikipedia.org/wiki/List_of_English_animal_nouns',

		],
		representation='table',
		columns={SPECIES: ['animal', 'animal_name', 'species_and_subspecies']}
	)}


def get_dinosaurs(wikipedia=None, echo=0):
	return {'dinosaur': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'dinosaur'],
		data_type=SPECIES,
		urls='https://en.wikipedia.org/wiki/List_of_dinosaur_genera',
		representation='list'
	)}


def get_extinct_mammals(wikipedia=None, echo=0):
	return {'extinct_mammals': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'mammal', 'extinct', 'extinct_mammal'],
		data_type=SPECIES,
		urls=[
			'https://en.wikipedia.org/wiki/List_of_recently_extinct_mammals',
			'https://en.wikipedia.org/wiki/List_of_extinct_cetaceans',
			'https://en.wikipedia.org/wiki/List_of_fossil_primates',
			'https://en.wikipedia.org/wiki/Category:Extinct_mammals'
		],
		columns={SPECIES: ['common_name']}
	)}


def get_extinct_birds(wikipedia=None, echo=0):
	return {'extinct_bird': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'bird', 'extinct', 'extinct_bird'],
		data_type=SPECIES,
		urls='https://en.wikipedia.org/wiki/List_of_recently_extinct_bird_species',
		representation='list'
	)}


def get_extinct_species(wikipedia=None, echo=0):
	return {'extinct_species': WikipediaListGuide(
		list_type='list_of_lists',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'extinct'],
		data_type=SPECIES,
		urls=[
			'https://en.wikipedia.org/wiki/Lists_of_extinct_animals',
			'https://en.wikipedia.org/wiki/Lists_of_extinct_species'
		],
		columns={SPECIES: ['common_name', 'scientific_name', SPECIES]}
	)}


def get_mammals(wikipedia=None, echo=0):
	return {
		'mammals': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'mammal'],
			data_type=SPECIES,
			urls=[
				'https://en.wikipedia.org/wiki/List_of_even-toed_ungulates_by_population',
				'https://en.wikipedia.org/wiki/List_of_carnivorans_by_population',
				'https://en.wikipedia.org/wiki/List_of_cetacean_species',
				'https://en.wikipedia.org/wiki/List_of_bats_by_population',
				'https://en.wikipedia.org/wiki/List_of_odd-toed_ungulates_by_population',
				'https://en.wikipedia.org/wiki/List_of_primates_by_population'
			],
			representation='table',
			columns={SPECIES: ['common_name']}
		),
		'primates': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'animal', 'mammal', 'primate'],
			data_type=SPECIES,
			urls=[
				'https://en.wikipedia.org/wiki/List_of_primates_by_population'
			],
			representation='table',
			columns={SPECIES: ['common_name']}
		)
	}


def get_plants(wikipedia=None, echo=0):
	return {'plants': WikipediaListGuide(
		list_type='list',
		wikipedia=wikipedia, echo=echo - 1, categories=[SPECIES, 'plant'],
		data_type=SPECIES,
		urls=[
			'https://en.wikipedia.org/wiki/Lists_of_cultivars'
		],
		columns={SPECIES: ['common_name']}
	)}


def get_species(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			**get_animals(wikipedia=wikipedia, echo=progress_bar),
			**get_domesticated_animals(wikipedia=wikipedia, echo=progress_bar),
			**get_herbivores(wikipedia=wikipedia, echo=progress_bar),
			**get_pests(wikipedia=wikipedia, echo=progress_bar),
			**get_mammals(wikipedia=wikipedia, echo=progress_bar),
			**get_dinosaurs(wikipedia=wikipedia, echo=progress_bar),
			**get_extinct_mammals(wikipedia=wikipedia, echo=progress_bar),
			**get_extinct_species(wikipedia=wikipedia, echo=progress_bar),
			**get_plants(wikipedia=wikipedia, echo=progress_bar),
			**get_extinct_birds(wikipedia=wikipedia, echo=progress_bar)
		}
	)


def get_breeds(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			'cat_breeds': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=echo - 1, categories=['breed', 'cat', 'cat_breed', 'animal', 'mammal'],
				data_type='breed',
				urls='https://en.wikipedia.org/wiki/List_of_cat_breeds',
				columns={'breed': ['breed']}
			),
			'dog_breeds': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=echo - 1, categories=['breed', 'dog', 'dog_breed', SPECIES, 'animal', 'mammal'],
				data_type='breed',
				urls='https://en.wikipedia.org/wiki/List_of_dog_breeds',
				columns={'breed': ['breed']}
			),
			'akc_breeds': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=echo - 1, categories=['breed', 'dog', 'dog_breed', SPECIES, 'animal', 'mammal'],
				data_type='breed',
				urls='https://en.wikipedia.org/wiki/American_Kennel_Club',
				representation='list',
				sections=['Recognized breeds']
			),
			'ckc_breeds': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=echo - 1, categories=['breed', 'dog', 'dog_breed', SPECIES, 'animal', 'mammal'],
				data_type='breed',
				urls='https://en.wikipedia.org/wiki/Canadian_Kennel_Club',
				representation='list'
			),
			'horse_breeds': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=echo - 1, categories=['breed', 'horse', 'horse_breed', SPECIES, 'animal', 'mammal'],
				data_type='breed',
				urls=[
					'https://en.wikipedia.org/wiki/List_of_horse_breeds',
					'https://en.wikipedia.org/wiki/List_of_gaited_horse_breeds',
					'https://en.wikipedia.org/wiki/List_of_Brazilian_horse_breeds',
					'https://en.wikipedia.org/wiki/List_of_French_horse_breeds',
					'https://en.wikipedia.org/wiki/List_of_Italian_horse_breeds'
				],
				representation='list'
			)
		}
	)



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar


POLITICAL_ENTITY = 'political_entity'
COUNTRY = 'country'
STATE = 'state'

def get_political_entity_guides(wikipedia=None, echo=0):
	return {
		'countries_and_dependencies_by_population': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, COUNTRY],
			data_type=POLITICAL_ENTITY,
			urls=[
				'https://en.wikipedia.org/wiki/List_of_countries_and_dependencies_by_population',
				'https://en.wikipedia.org/wiki/List_of_national_capitals'

			],
			columns={POLITICAL_ENTITY: ['country_or_dependent_territory', 'country_dependent_territory', 'country']}
		),
		'commonwealth_of_nations': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, COUNTRY, 'commonwealth'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/Member_states_of_the_Commonwealth_of_Nations',
			columns={POLITICAL_ENTITY: ['country']}
		),
		'national_capitals': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city', 'capital'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/List_of_national_capitals',
			columns={POLITICAL_ENTITY: 'city'}
		),
		'states_and_territories_of_the_united_states': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, STATE, 'united_states'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/List_of_states_and_territories_of_the_United_States_by_population',
			columns={POLITICAL_ENTITY: 'name'}
		),
		'states_of_mexico': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, STATE, 'mexico'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/List_of_states_of_Mexico',
			columns={POLITICAL_ENTITY: 'state'}
		),
		'cities_by_country': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city'],
			data_type=POLITICAL_ENTITY,
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_cities_by_country',
			],
			columns={POLITICAL_ENTITY: [
				'name', 'city_town', 'city', 'cities_towns', 'name_of_town_or_city', 'city_localidad', 'city_or_town',
				'cities', 'common_name', 'municipality', 'gccsa_sua', 'urban_area', 'municipio',
				'belarusian_bgn_pcgn', 'capital', 'english', 'settlement',
				'local_council'
			]}
		),
		'united_states_cities': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city', 'united_states'],
			data_type=POLITICAL_ENTITY,
			urls=[
				'https://en.wikipedia.org/wiki/List_of_United_States_cities_by_population',
				'https://en.wikipedia.org/wiki/List_of_cities_and_towns_in_California',
				'https://en.wikipedia.org/wiki/List_of_cities_and_towns_in_Washington',
				'https://en.wikipedia.org/wiki/List_of_cities_and_towns_in_Arizona',
				'https://en.wikipedia.org/wiki/List_of_cities_in_West_Virginia',
				'https://en.wikipedia.org/wiki/List_of_cities_in_New_York_(state)'
			],
			columns={POLITICAL_ENTITY: [
				'name', 'city_town', 'city', 'cities_towns', 'name_of_town_or_city', 'city_localidad', 'city_or_town',
				'cities', 'common_name', 'municipality', 'gccsa_sua', 'urban_area', 'municipio',
				'belarusian_bgn_pcgn', 'capital', 'english', 'settlement',
				'local_council'
			]}
		),
		'municipalities_of_brazil': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city', 'municipality', 'brazil'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/Municipalities_of_Brazil',
			columns={POLITICAL_ENTITY: ['municipality']}
		),
		'towns_in_india': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city', 'indian_town', 'india'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/List_of_towns_in_India_by_population',
			columns={POLITICAL_ENTITY: 'name_of_town'}
		),
		'united_kingdom_locations': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'place', 'place_in_united_kingdom', 'united_kingdom'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/List_of_United_Kingdom_locations',
			columns={POLITICAL_ENTITY: ['location']}
		),
		'cities_in_germany': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city', 'german_city', 'germany'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/List_of_cities_in_Germany_by_population',
			columns={POLITICAL_ENTITY: 'city'}
		),
		'cities_in_italy': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city', 'italian_city', 'italy'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/List_of_cities_in_Italy',
			columns={POLITICAL_ENTITY: 'city'}
		),
		'coastal_settlements_of_the_mediterranean_sea': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'city'],
			data_type=POLITICAL_ENTITY,
			urls=[
				'https://en.wikipedia.org/wiki/List_of_coastal_settlements_of_the_Mediterranean_Sea',
				'https://en.wikipedia.org/wiki/List_of_North_Sea_ports',
				'https://en.wikipedia.org/wiki/List_of_cities_and_towns_in_Romania',
				'https://en.wikipedia.org/wiki/List_of_cities_in_Ukraine',
				'https://en.wikipedia.org/wiki/List_of_largest_cities_in_Brazil',
				'https://en.wikipedia.org/wiki/List_of_cities_in_Nepal'
			],
			columns={POLITICAL_ENTITY: [
				'name', 'city_town', 'city', 'cities_towns', 'name_of_town_or_city', 'city_localidad', 'city_or_town',
				'cities', 'common_name', 'municipality', 'gccsa_sua', 'urban_area', 'municipio',
				'belarusian_bgn_pcgn', 'capital', 'english', 'settlement',
				'local_council'
			]}
		),
		'counties_in_the_united_states': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, categories=[POLITICAL_ENTITY, 'county', 'united_states'],
			data_type=POLITICAL_ENTITY,
			urls='https://en.wikipedia.org/wiki/Lists_of_counties_in_the_United_States',
			columns={POLITICAL_ENTITY: ['county', 'county_or_equivalent']}
		)
	}


def get_political_entities(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)

	return WikipediaLists(
		wikipedia_lists=get_political_entity_guides(wikipedia=wikipedia, echo=progress_bar),
		echo=progress_bar
	)


from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

VEHICLE = 'vehicle'


def get_vehicles(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)

	aircrafts = {
		'civil_aircrafts': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft', 'civil_aircraft'],
			urls='https://en.wikipedia.org/wiki/List_of_civil_aircraft',
			representation='list'
		),
		'bush_planes': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft', 'bush_plane'],
			urls='https://en.wikipedia.org/wiki/Bush_plane',
			representation='list',
			sections=['Current and historical bush planes']
		),
		'light_transport_aircrafts': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft', 'transport_aircraft', 'light_transport_aircraft'],
			urls='https://en.wikipedia.org/wiki/List_of_light_transport_aircraft',
			representation='table',
			columns={VEHICLE: ['model']}
		),
		'racing_aircrafts': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_racing_aircraft',
				'https://en.wikipedia.org/wiki/List_of_STOL_aircraft',
				'https://en.wikipedia.org/wiki/List_of_VTOL_aircraft',
				'https://en.wikipedia.org/wiki/List_of_seaplanes_and_amphibious_aircraft'
			],
			representation=None,
			columns={VEHICLE: ['type']}
		),
		'fighter_aircrafts': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft', 'military_vehicle', 'military_aircraft', 'fighter_aircraft'],
			urls='https://en.wikipedia.org/wiki/List_of_fighter_aircraft',
			representation=None,
			columns={VEHICLE: ['type']}
		),
		'bomber_aircrafts': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft', 'military_vehicle', 'military_aircraft', 'bomber_aircraft'],
			urls='https://en.wikipedia.org/wiki/List_of_bomber_aircraft',
			representation='table',
			columns={VEHICLE: ['type']}
		),
		'us_military_aircrafts': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft', 'military_vehicle', 'military_aircraft', 'united_states_aircraft'],
			urls='https://en.wikipedia.org/wiki/List_of_active_United_States_military_aircraft',
			representation=None,
			columns={VEHICLE: ['type']}
		),
		'military_aircrafts': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'aircraft', 'military_vehicle', 'military_aircraft'],
			urls='https://en.wikipedia.org/wiki/Category:Lists_of_military_aircraft',
			representation=None,
			columns={VEHICLE: ['type', 'aircraft']},
			sections=['Pages in category']
		)
	}

	ships = {
		'ships': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_cargo_ships',
				'https://en.wikipedia.org/wiki/Nuclear_marine_propulsion#Civilian_nuclear_ships',
				'https://en.wikipedia.org/wiki/List_of_cruise_ships',
				'https://en.wikipedia.org/wiki/List_of_river_cruise_ships',
				'https://en.wikipedia.org/wiki/List_of_largest_ferries_of_Europe',
				'https://en.wikipedia.org/wiki/List_of_icebreakers'
			],
			representation=None,
			columns={VEHICLE: ['name', 'ship']}
		),
		'liberty_ships': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'liberty_ship'],
			urls='https://en.wikipedia.org/wiki/List_of_Liberty_ships',
			representation='table', columns={VEHICLE: 'ship_name'}
		),
		'us_navy_ships': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'military_ship', 'united_states_navy', 'united_states_navy_ship'],
			urls='https://en.wikipedia.org/wiki/List_of_current_ships_of_the_United_States_Navy',
			representation='table',
			columns={VEHICLE: ['ship_name']}
		),
		'royal_canadian_navy_ships': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'military_ship', 'royal_canadian_navy', 'royal_canadian_navy_ship'],
			urls='https://en.wikipedia.org/wiki/List_of_ships_of_the_Royal_Canadian_Navy',
			representation='list'
		),
		'battleships': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'military_ship', 'battleship'],
			urls='https://en.wikipedia.org/wiki/List_of_battleships',
			representation='table',
			columns={VEHICLE: ['name']}
		),
		'us_navy_ship_classes': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'ship_class'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_current_ships_of_the_United_States_Navy',
				'https://en.wikipedia.org/wiki/List_of_battleships',
				'https://en.wikipedia.org/wiki/List_of_ships_of_the_Egyptian_Navy'
			],
			representation='table',
			columns={VEHICLE: ['class']}
		),
		'us_navy_ship_types': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'ship_type'],
			urls='https://en.wikipedia.org/wiki/List_of_current_ships_of_the_United_States_Navy',
			representation='table',
			columns={VEHICLE: ['type']}
		),
		'ocean_liners': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'ocean_liner'],
			urls='https://en.wikipedia.org/wiki/List_of_ocean_liners',
			representation='table',
			columns={VEHICLE: ['ship_name']}
		),
		'aircraft_carriers': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'ship', 'military_ship', 'aircraft_carrier'],
			urls='https://en.wikipedia.org/wiki/List_of_aircraft_carriers',
			representation='list'
		)
	}

	military_vehicles = {
		'military_vehicles': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'combat_vehicle', 'military_vehicle'],
			urls='https://en.wikipedia.org/wiki/List_of_U.S._military_vehicles_by_model_number',
			representation='list'
		)
	}

	cars1 = {
		'bmw_cars': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'car', 'bmw'],
			urls='https://en.wikipedia.org/wiki/List_of_BMW_vehicles',
			representation='list',
			sections=['Cars']
		),
		'bmw_motorcycles': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'motorcycle', 'bmw'],
			urls='https://en.wikipedia.org/wiki/List_of_BMW_vehicles',
			representation='list',
			sections=['Motorcycles']
		),
		'automobiles': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'car'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_best-selling_automobiles',
				'https://en.wikipedia.org/wiki/List_of_automobile_sales_by_model'
			],
			representation='table',
			columns={VEHICLE: ['automobile']}
		),
		'toyota_cars': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'toyota', 'car'],
			urls='https://en.wikipedia.org/wiki/List_of_Toyota_vehicles',
			representation='table',
			columns={VEHICLE: ['model_2']}
		)
	}

	brands_and_urls = {
		'audi': 'https://en.wikipedia.org/wiki/List_of_Audi_vehicles',
		'austin': 'https://en.wikipedia.org/wiki/List_of_Austin_motor_vehicles',
		'ford': 'https://en.wikipedia.org/wiki/List_of_Ford_vehicles',
		'mercury': 'https://en.wikipedia.org/wiki/List_of_Mercury_vehicles',
		'moskvitch': 'https://en.wikipedia.org/wiki/List_of_Moskvitch_vehicles',
		'geo': 'https://en.wikipedia.org/wiki/List_of_Geo_vehicles',
		'buich': 'https://en.wikipedia.org/wiki/List_of_Buick_vehicles',
		'honda': 'https://en.wikipedia.org/wiki/List_of_Honda_automobiles',
		'chevrolet': 'https://en.wikipedia.org/wiki/List_of_Chevrolet_vehicles',
		'chrysler': 'https://en.wikipedia.org/wiki/List_of_Chrysler_vehicles',
		'diesel_car': 'https://en.wikipedia.org/wiki/List_of_diesel_automobiles',
		'dodge': 'https://en.wikipedia.org/wiki/List_of_Dodge_vehicles',
		'electric_car': [
			'https://en.wikipedia.org/wiki/List_of_electric_cars_currently_available',
			'https://en.wikipedia.org/wiki/List_of_production_battery_electric_vehicles_(table)'
		],
		'battery_electric_car': 'https://en.wikipedia.org/wiki/List_of_production_battery_electric_vehicles',
		'european_car': 'https://en.wikipedia.org/wiki/List_of_European_automobiles',
		'fast_car': 'https://en.wikipedia.org/wiki/List_of_fastest_production_cars_by_acceleration',
		'iran_khodro': 'https://en.wikipedia.org/wiki/List_of_Iran_Khodro_vehicles',
		'kia': 'https://en.wikipedia.org/wiki/List_of_Kia_Motors_automobiles',
		'lambodghini': 'https://en.wikipedia.org/wiki/List_of_Lamborghini_automobiles',
		'le_mans_prototype': 'https://en.wikipedia.org/wiki/List_of_Le_Mans_Prototypes',
		'lexus': 'https://en.wikipedia.org/wiki/List_of_Lexus_vehicles',
		'dacia': 'https://en.wikipedia.org/wiki/List_of_Dacia_vehicles',
		'gmc': 'https://en.wikipedia.org/wiki/List_of_GMC_vehicles',
		'hyundai': 'https://en.wikipedia.org/wiki/List_of_Hyundai_vehicles',
		'lincoln': 'https://en.wikipedia.org/wiki/List_of_Lincoln_vehicles',
		'van': 'https://en.wikipedia.org/wiki/List_of_vans',
		'maserati': 'https://en.wikipedia.org/wiki/List_of_Maserati_vehicles',
		'mazda': 'https://en.wikipedia.org/wiki/List_of_Mazda_vehicles',
		'mercedes_benz': 'https://en.wikipedia.org/wiki/List_of_Mercedes-Benz_vehicles',
		'oldsmobile': 'https://en.wikipedia.org/wiki/List_of_Oldsmobile_vehicles',
		'opel': 'https://en.wikipedia.org/wiki/List_of_Opel_vehicles',
		'packard': 'https://en.wikipedia.org/wiki/Packard#Packard_automobile_models',
		'plymouth': 'https://en.wikipedia.org/wiki/List_of_Plymouth_vehicles',
		'pontiac': 'https://en.wikipedia.org/wiki/List_of_Pontiac_vehicles',
		'porsche': 'https://en.wikipedia.org/wiki/List_of_Porsche_vehicles',
		'renault': 'https://en.wikipedia.org/wiki/List_of_Renault_vehicles',
		'saab': 'https://en.wikipedia.org/wiki/List_of_Saab_passenger_cars',
		'suv': 'https://en.wikipedia.org/wiki/List_of_sport_utility_vehicles',
		'sports_car': 'https://en.wikipedia.org/wiki/List_of_sports_cars',
		'vauxhall': 'https://en.wikipedia.org/wiki/List_of_Vauxhall_vehicles',
		'volkswagen': 'https://en.wikipedia.org/wiki/List_of_Volkswagen_passenger_vehicles',
		'volvo': 'https://en.wikipedia.org/wiki/List_of_Volvo_passenger_cars',
		'zil': 'https://en.wikipedia.org/wiki/List_of_ZiL_vehicles'
	}

	cars2 = {
		brand: WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=VEHICLE,
			categories=[VEHICLE, 'car', brand],
			urls=url,
			columns={VEHICLE: ['name', 'model', 'model_name', 'car', 'vehicle_name_with_photograph', 'suv']},
			representation=None
		)
		for brand, url in brands_and_urls.items()
	}

	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			**aircrafts, **ships, **military_vehicles, **cars1, **cars2
		}
	)


from numpy import nan

class Country:
	def __init__(self, dictionary):
		"""
		:type dictionary: dict
		"""
		self._alpha_2 = None
		self._alpha_3 = None
		self._common_name = None
		self._name = None
		self._numeric = None
		self._official_name = None
		self._match_score = 0
		for k, v in dictionary.items():
			try:
				key = f'_{k}'
				if v is nan: v=None
				setattr(self, key, v)
			except:
				raise RuntimeError(f'cannot set key:"{key}" to value:"{v}"')
	@property
	def alpha2(self):
		return self._alpha_2

	code = alpha2

	@property
	def alpha3(self):
		return self._alpha_3

	@property
	def common_name(self):
		return self._common_name

	@property
	def name(self):
		return self._name

	@property
	def numeric(self):
		return self._numeric

	@property
	def official_name(self):
		return self._official_name

	@property
	def match_score(self):
		return self._match_score

	def __repr__(self):
		return str(self)

	def __str__(self):
		return f'Country: {self.name} [{self.code}] [{self.match_score}]'





from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

PRODUCT = 'product'


def get_products(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(wikipedia_lists={
		'books': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'book'],
			urls='https://en.wikipedia.org/wiki/Lists_of_books',
			columns={PRODUCT: ['title', 'book_title']},
			sections=['General lists', 'Writer lists', 'Series lists'],
			representation=None
		),
		'drugs': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'drug'],
			urls='https://en.wikipedia.org/wiki/List_of_drugs_by_year_of_discovery',
			columns={PRODUCT: ['name_of_the_drug']},
			representation='table'
		),
		'software': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'intangible_good', 'software', 'web_browser'],
			urls='https://en.wikipedia.org/wiki/List_of_web_browsers',
			columns={PRODUCT: ['browser', 'web_browsers']},
			sections=['Historical', 'Graphical', 'Text-based']
		),
		'programming_languages': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'intangible_good', 'programming_language'],
			urls='https://en.wikipedia.org/wiki/List_of_programming_languages',
			representation='list'
		),
		'video_games': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'intangible_good', 'video_game'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_video_games_considered_the_best',
				'https://en.wikipedia.org/wiki/List_of_Xbox_games',
				'https://en.wikipedia.org/wiki/List_of_Xbox_360_games',
				'https://en.wikipedia.org/wiki/List_of_PC_games',
				'https://en.wikipedia.org/wiki/List_of_grand_strategy_video_games',
				'https://en.wikipedia.org/wiki/List_of_city-building_video_games',
				'https://en.wikipedia.org/wiki/List_of_business_simulation_video_games',
				'https://en.wikipedia.org/wiki/List_of_real-time_strategy_video_games',
				'https://en.wikipedia.org/wiki/List_of_god_video_games',
				'https://en.wikipedia.org/wiki/List_of_first-person_shooters',
				'https://en.wikipedia.org/wiki/List_of_driving_and_racing_video_games',
				'https://en.wikipedia.org/wiki/List_of_roguelikes',
				'https://en.wikipedia.org/wiki/List_of_space_flight_simulator_games',
				'https://en.wikipedia.org/wiki/List_of_Macintosh_games',
				'https://en.wikipedia.org/wiki/List_of_4X_video_games',
				'https://en.wikipedia.org/wiki/List_of_games_using_Havok',
				'https://en.wikipedia.org/wiki/List_of_zombie_video_games',
				'https://en.wikipedia.org/wiki/List_of_third-person_shooters',
				'https://en.wikipedia.org/wiki/List_of_music_video_games',
				'https://en.wikipedia.org/wiki/List_of_artillery_video_games',
				'https://en.wikipedia.org/wiki/List_of_BattleTech_games',
				'https://en.wikipedia.org/wiki/List_of_turn-based_strategy_video_games',
				'https://en.wikipedia.org/wiki/List_of_real-time_tactics_video_games',
				'https://en.wikipedia.org/wiki/List_of_massively_multiplayer_online_strategy_video_games',
				'https://en.wikipedia.org/wiki/List_of_turn-based_tactics_video_games',
				'https://en.wikipedia.org/wiki/List_of_PlayStation_4_games'
			],
			columns={PRODUCT: ['game', 'title', 'title_and_source']},
			sections=['List']
		),
		'more_video_games': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, categories=[PRODUCT, 'intangible_good', 'video_game'],
			data_type=PRODUCT,
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_PlayStation_3_games',
				'https://en.wikipedia.org/wiki/Lists_of_Nintendo_games'
			],
			columns={PRODUCT: ['game', 'title', 'title_and_source', 'original_title']}
		),
		'consumer_electronics': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, categories=[PRODUCT, 'consumer_electronic'], data_type=PRODUCT,
			urls=[
				'https://en.wikipedia.org/wiki/Timeline_of_Apple_Inc._products'
			],
			columns={PRODUCT: ['model']}
		),
		'beatles_songs': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'intangible_good', 'music', 'song', 'by_the_beatles'],
			urls='https://en.wikipedia.org/wiki/List_of_songs_recorded_by_the_Beatles',
			columns={PRODUCT: ['song']}
		),
		'beatles_albums': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'intangible_good', 'music', 'music_album', 'by_the_beatles'],
			urls='https://en.wikipedia.org/wiki/List_of_songs_recorded_by_the_Beatles',
			columns={PRODUCT: ['core_catalogue_release', 'release_s']}
		),
		'tv_programs': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, categories=[PRODUCT, 'intangible_good', 'tv_program'],
			data_type=PRODUCT,
			urls=[
				'https://en.wikipedia.org/wiki/List_of_television_programmes_broadcast_by_the_BBC',
				'https://en.wikipedia.org/wiki/List_of_British_television_programmes',
				'https://en.wikipedia.org/wiki/List_of_programs_broadcast_by_HBO',
				'https://en.wikipedia.org/wiki/List_of_programs_broadcast_by_GMA_Network',
				'https://en.wikipedia.org/wiki/List_of_programs_broadcast_by_ABS-CBN'
			],
			columns={PRODUCT: ['title']}
		),
		'movies': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, categories=[PRODUCT, 'intangible_good', 'movie'], data_type=PRODUCT,
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_films',
				'https://en.wikipedia.org/wiki/Lists_of_anime'
			], sections=['Alphabetical indices', 'By year'],
			columns={PRODUCT: ['title', 'english_name']}
		),
		'indian_movies': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1,
			categories=[PRODUCT, 'intangible_good', 'movie', 'bollywood_movie', 'indian'], data_type=PRODUCT,
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_Bollywood_films'
			], sections=['Alphabetical indices', 'By year'],
			columns={PRODUCT: ['title', 'english_name']}
		),
		'american_movies': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1,
			categories=[PRODUCT, 'intangible_good', 'movie', 'american_movie', 'american'],
			data_type=PRODUCT,
			urls='https://en.wikipedia.org/wiki/Lists_of_American_films',
			columns={PRODUCT: ['title']}
		),
		'medicines': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, categories=[PRODUCT, 'tangible_good', 'medicine'],
			data_type=PRODUCT,
			urls='https://en.wikipedia.org/wiki/WHO_Model_List_of_Essential_Medicines',
			representation='list'
		),
		'journals': WikipediaListGuide(
		list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT, categories=[PRODUCT, 'journal'],
			urls=[
				'https://en.wikipedia.org/wiki/List_of_journals',
				'https://en.wikipedia.org/wiki/Lists_of_academic_journals'
			],
			exclude_urls=[
				'https://en.wikipedia.org/wiki/Lists_of_academic_journals',
				'https://en.wikipedia.org/wiki/Slavic_studies',
				'https://en.wikipedia.org/wiki/Arachnology'
			],
			columns={PRODUCT: ['journal', 'journal_name', 'name', 'title']}
		),
		'websites': WikipediaListGuide(
		list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'intangible_good', 'website'],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_websites',
				'https://en.wikipedia.org/wiki/List_of_search_engines'
			],
			exclude_urls=[
				'https://en.wikipedia.org/wiki/Lists_of_webcomics',
				'https://en.wikipedia.org/wiki/Health_information_on_the_Internet',
				'https://en.wikipedia.org/wiki/List_of_websites_blocked_in_Russia'
			],
			columns={PRODUCT: ['site', 'name', 'blog', 'website', 'site_name']}
		),
		'social_networking_websites': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'intangible_good', 'website'],
			urls='https://en.wikipedia.org/wiki/List_of_social_networking_websites',
			columns={PRODUCT: ['name']}
		),
		'foods': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=progress_bar - 1, data_type=PRODUCT,
			categories=[PRODUCT, 'tangible_good', 'food'],
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_foods'
			],
			representation='list'
		)
	})



from internet import Wikipedia


def get_text(x):
	try:
		return x.text
	except:
		return str(x)


def get_gics_table(url='https://en.wikipedia.org/wiki/Global_Industry_Classification_Standard', table_num=0):
	wikipedia = Wikipedia()
	page = wikipedia.get_page(url=url)
	table = page.tables[table_num].rename(columns={
		'sector_1': 'sector_code', 'sector_2': 'sector',
		'industry_group_1': 'industry_group_code', 'industry_group_2': 'industry_group',
		'industry_1': 'industry_code', 'industry_2': 'industry',
		'sub_industry_1': 'sub_industry_code', 'sub_industry_2': 'sub_industry'
	})
	for column in table.columns:
		table[column] = table[column].apply(func=get_text)
	return table


GICS_TABLE = get_gics_table()

def to_dict(df):
	return dict(sorted(df.values.tolist()))

SECTORS = to_dict(GICS_TABLE[['sector_code', 'sector']].drop_duplicates())
INDUSTRY_GROUPS = to_dict(GICS_TABLE[['industry_group_code', 'industry_group']].drop_duplicates())
INDUSTRIES = to_dict(GICS_TABLE[['industry_code', 'industry']].drop_duplicates())
SUB_INDUSTRIES = to_dict(GICS_TABLE[['sub_industry_code', 'sub_industry']].drop_duplicates())


def get_gics(code):
	code = str(code).ljust(8)
	sector = SECTORS.get(code[:2], None)
	industry_group = INDUSTRY_GROUPS.get(code[:4], sector)
	industry = INDUSTRIES.get(code[:6], industry_group)
	sub_industry = SUB_INDUSTRIES.get(code[:8], industry)
	return {
		'code': code, 'sector': sector, 'industry_group': industry_group,
		'industry': industry, 'sub_industry': sub_industry
	}



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

BRAND = 'brand'
BRAND_OR_COMPANY = 'brand_or_company'

def get_brand_list_guides(wikipedia=None, echo=0):
	brand_lists = {
		'car_brands': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'car_brand'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_car_brands'
		),
		'car_brands_2': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'car_brand'],
			representation='table',
			sections=['Brand bestsellers'],
			columns={BRAND: ['brand']},
			urls='https://en.wikipedia.org/wiki/List_of_best-selling_automobiles'
		),
		'procter_and_gamble': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'procter_and_gamble'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Procter_%26_Gamble_brands'
		),
		'unilever': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'unilever'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Unilever_brands'
		),
		'cocacola': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'cocacola'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Coca-Cola_brands'
		),
		'keurig': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'keurig'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Keurig_Dr_Pepper_brands'
		),
		'molson': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'molson'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Molson_Coors_brands'
		),
		'pepsi': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'pepsi'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Pepsi_variations'
		),
		'conagra': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'conagra'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Conagra_brands'
		),
		'kraft': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'kraft'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Kraft_brands'
		),
		'nestle': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'nestle'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_Nestl%C3%A9_brands'
		),
		'british_rail': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'british_rail'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/British_Rail_brand_names'
		),
		'toy': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'toy'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_toys'
		),
		'toothpaste': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'toothpaste'],
			representation='list',
			urls='https://en.wikipedia.org/wiki/List_of_toothpaste_brands'
		),
		'rolling_paper': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'rolling_paper'],
			representation='table', columns={BRAND: 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_rolling_papers'
		),
		'fitness_wear': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'fitness_wear'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_fitness_wear_brands'
		),
		'japanese_bicycle': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'japanese_bicycle'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_Japanese_bicycle_brands_and_manufacturers'
		),
		'molson_coors': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'molson_coors'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_Molson_Coors_brands'
		),
		'rum': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'rum'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_rum_producers'
		),
		'soft_drink': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'soft_drink'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_soft_drinks_by_country'
		),
		'tequila': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'tequila'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_tequilas'
		),
		'whisky': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'whisky'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_whisky_brands'
		),
		'bread': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'bread'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_brand_name_breads'
		),
		'confectionery': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'confectionery'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_confectionery_brands'
		),
		'food': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'food'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_brand_name_food_products'
		),
		'frozen_dessert': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'frozen_dessert'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_frozen_dessert_brands'
		),
		'mustard': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'mustard'],
			representation='list', 
			urls='https://en.wikipedia.org/wiki/List_of_mustard_brands'
		),
		'lingerie': WikipediaListGuide(
			list_type='list', data_type=BRAND_OR_COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'lingerie'],
			representation='table', columns={BRAND_OR_COMPANY: 'brand'},
			urls='https://en.wikipedia.org/wiki/List_of_lingerie_brands'
		),
		'cider': WikipediaListGuide(
			list_type='list', data_type=BRAND_OR_COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'cider'],
			representation='table', columns={BRAND_OR_COMPANY: 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_cider_brands'
		),
		'candy': WikipediaListGuide(
			list_type='list', data_type=BRAND_OR_COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'candy'],
			representation='table', columns={BRAND_OR_COMPANY: 'top_brands'},
			urls='https://en.wikipedia.org/wiki/List_of_top-selling_candy_brands'
		),
		'popcorn': WikipediaListGuide(
			list_type='list', data_type=BRAND_OR_COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND, 'popcorn'],
			representation='table', columns={BRAND_OR_COMPANY: 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_popcorn_brands'
		),
		'firearm': WikipediaListGuide(
			list_type='list', data_type=BRAND_OR_COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND_OR_COMPANY, 'firearm'],
			representation='table', columns={'company': 'company', 'brand': 'marque'},
			urls='https://en.wikipedia.org/wiki/List_of_firearm_brands'
		),
		'perfume': WikipediaListGuide(
			list_type='list', data_type=BRAND_OR_COMPANY, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND_OR_COMPANY, 'perfume'],
			representation='table', columns={'company': 'company', 'brand': 'name'},
			urls='https://en.wikipedia.org/wiki/List_of_perfumes'
		),
		'vodka': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND_OR_COMPANY, 'vodka'],
			representation='table', columns={BRAND: 'brand'},
			urls='https://en.wikipedia.org/wiki/List_of_vodkas'
		),
		'breath_mint': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND_OR_COMPANY, 'breath_mint'],
			representation='table', columns={BRAND: 'name', 'company': 'company'},
			urls='https://en.wikipedia.org/wiki/List_of_breath_mints'
		),
		'chewing_gum': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND_OR_COMPANY, 'chewing_gum'],
			representation='table', columns={BRAND: 'name', 'company': 'parent_company'},
			urls='https://en.wikipedia.org/wiki/List_of_chewing_gum_brands'
		),
		'chocolate_bar': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND_OR_COMPANY, 'chocolate_bar'],
			representation='table', columns={BRAND: 'name', 'company': 'manufacturer'},
			urls='https://en.wikipedia.org/wiki/List_of_chocolate_bar_brands'
		),
		'instant_noodle': WikipediaListGuide(
			list_type='list', data_type=BRAND, echo=echo - 1,
			wikipedia=wikipedia, categories=[BRAND_OR_COMPANY, 'instant_noodle'],
			representation='table', columns={BRAND: 'name', 'company': 'current_owner'},
			urls='https://en.wikipedia.org/wiki/List_of_instant_noodle_brands'
		)
	}
	return brand_lists


def get_brands(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	list_guides = get_brand_list_guides(wikipedia=wikipedia, echo=progress_bar)
	return WikipediaLists(wikipedia_lists=list_guides, categories=[BRAND], echo=progress_bar)



from internet.wikipedia import Page
from copy import deepcopy
from functools import wraps
from silverware import Spoon

class SpecialPage(Page):
	@wraps(Page.__init__)
	def __init__(self, url, wikipedia=None, page=None, sections=None):
		if page is None:
			super().__init__(wikipedia=wikipedia, url=url)
		else:
			self._wikipedia = page.api
			self._pensieve = deepcopy(page.pensieve)
		self.pensieve['section_of_interest_names'] = sections

		def _get_sections_of_interest(body, section_of_interest_names):
			if section_of_interest_names is None:

				the_end = body.find(name='a', attrs={'href': 'http://SEEALSO'})
				if the_end is None:
					_sections = Spoon(soup=body)
				else:
					_sections = Spoon.before(element=the_end)
			else:
				try:
					_sections = Spoon.get_sections(sections=section_of_interest_names, soup=body)
				except KeyError as e:
					_sections = _get_sections_of_interest(body=body, section_of_interest_names=None)

			return _sections

		self.pensieve.store(
			key='sections_of_interest', precursors=['body', 'section_of_interest_names'],
			function=lambda x: _get_sections_of_interest(
				body=x['body'], section_of_interest_names=x['section_of_interest_names']
			),
			evaluate=False
		)



from .tokenize import tokenize


def get_header_tokens(headers):
	tokens = []
	for header in headers:
		text = header.text
		if text == 'SEEALSO':
			break
		text = text[:-6] if text.endswith('[edit]') else text
		tokens += tokenize(text)
	return tokens


from .lists.political_entities import get_political_entities
from .lists.space_objects import get_space_objects
from .lists.brands import get_brands
from .lists.companies import get_companies
from .lists.concepts import get_concepts
from .lists.diseases import get_diseases
from .lists.fictional_characters import get_fictional_characters
from .lists.geographical_features import get_geographical_features
from .lists.historic_events import get_historic_events
from .lists.landmarks import get_landmarks
from .lists.music_bands import get_music_bands
from .lists.organizations import get_organizations
from .lists.persons import get_persons
from .lists.products import get_products
from .lists.species import get_species, get_breeds
from .lists.supernatural_beings import get_supernatural_beings
from .lists.vehicles import get_vehicles
from .lists.weapons import get_weapons

from .signature.PageSignature import PageSignature

from disk import Cache
from chronometry.progress import ProgressBar, iterate
from pandas import concat
from math import ceil
from pensieve import Pensieve
from joblib import delayed, Parallel


LIST_FUNCTIONS = {
	'brand': get_brands,
	'company': get_companies,
	'concept': get_concepts,
	'disease': get_diseases,
	'fictional_character': get_fictional_characters,
	'geographical_feature': get_geographical_features,
	'historic_event': get_historic_events,
	'landmark': get_landmarks,
	'artist_or_band': get_music_bands,
	'organization': get_organizations,
	'person': get_persons,
	'political_entity': get_political_entities,
	'product': get_products,
	'space_object': get_space_objects,
	'specie': get_species,
	'breed': get_breeds,
	'supernatural_being': get_supernatural_beings,
	'vehicle': get_vehicles,
	'weapon': get_weapons
}


def get_data(entity_type, wikipedia, echo):
	wikipedia_list = LIST_FUNCTIONS[entity_type](wikipedia=wikipedia, echo=echo)
	return wikipedia_list.data

KEYS = sorted(LIST_FUNCTIONS.keys())


class WikipediaEntities:
	def __init__(
			self, sample_size=10000, different_type_ratio=1 / 10,
			wikipedia=None, num_threads=-1,
			echo=3, cache=None, random_state=42
	):
		self._num_threads = num_threads
		self._wikipedia = wikipedia
		self._echo = echo
		self._cache = cache
		self._create_cached_functions()
		self._pensieve = Pensieve()
		self._all_data = None

		self.pensieve.store(
			key='all_data', function=lambda :self.get_all(echo=echo),
			evaluate=False
		)
		self.pensieve['random_state'] = random_state
		self.pensieve['sample_size'] = sample_size
		self.pensieve['different_type_ratio'] = different_type_ratio
		self.pensieve.store(
			key='sample',
			function=lambda all_data, sample_size, different_type_ratio, random_state:
				self.get_entity_sample(
					size=sample_size, ratio=different_type_ratio, data=all_data,
					random_state=random_state, echo=echo
				),
			evaluate=False
		)




		def add_signatures(sample, echo):
			result = {}
			for key, data in sample.items():
				print(key)
				new_data = data.copy()
				urls = list(new_data['url'])
				progress_bar = ProgressBar(total=len(urls) + 1, echo=echo)
				signatures = self.processor(
					delayed(self.get_page_signature)(url)
					for url in iterate(
						iterable=urls, progress_bar=progress_bar, text=f'collecting {key} signatures'
					)
				)
				progress_bar.show(amount=len(urls) + 1, text='signatures_collected')
				new_data['signature'] = signatures

				result[key] = new_data
			return result
		self.pensieve.store(
			key='signatures', function=lambda sample: add_signatures(sample=sample, echo=echo),
			evaluate=False
		)

	@property
	def processor(self):
		return Parallel(n_jobs=self._num_threads, backend='threading', require='sharedmem')

	@property
	def pensieve(self):
		"""
		:rtype: Pensieve
		"""
		return self._pensieve

	_STATE_ATTRIBUTES_ = ['_wikipedia', '_echo', '_cache']

	def __getstate__(self):
		return {key: getattr(self, key) for key in self._STATE_ATTRIBUTES_}

	def __setstate__(self, state):
		for key, value in state.items():
			setattr(self, key, value)
		self._create_cached_functions()

	def __hashkey__(self):
		return self.__class__.__name__, self._wikipedia, self._cache

	def _get_page_signature(self, url):
		if url.startswith('https://en.wikipedia.org/wiki/') and \
				'#' not in url and 'index.php?' not in url and '/wiki/file:' not in url.lower():
			try:
				signature = PageSignature(wikipedia=self._wikipedia, url=url)
				return signature.signature
			except Exception as e:
				return {'error': str(e)}
		else:
			return {'error': 'bad_url'}

	@property
	def cache(self):
		"""
		:rtype: Cache
		"""
		return self._cache

	def _create_cached_functions(self):
		if self.cache:
			self.get_data = self.cache.make_cached(
				function=self._get_data, id='WikipediaEntities.get_data',
				sub_directory='entities_data', exclude_kwargs=['echo']
			)
			self.get_page_signature = self.cache.make_cached(
				function=self._get_page_signature, id='WikipediaEntities.get_page_signature',
				sub_directory='entities_signature',
				condition_function=lambda result: 'error' in result and len(result) < 2
			)

		else:
			self.get_data = self._get_data
			self.get_page_signature = self._get_page_signature

	def _get_data(self, entity_type, echo=0):
		return get_data(entity_type=entity_type, wikipedia=self._wikipedia, echo=echo)

	@property
	def types(self):
		return list(KEYS)

	def keys(self):
		return self.types

	def __getitem__(self, item):
		return self.get_data(entity_type=item, echo=0)

	def get_all(self, echo=1):
		"""
		:type echo: bool or int or ProgressBar
		:rtype: pandas.DataFrame
		"""
		if self._all_data is None:
			echo=max(0, echo)
			result = []
			progress_bar = ProgressBar(total=len(self.types), echo=echo)
			progress = 0
			for entity_type in self.types:
				progress_bar.show(amount=progress, text=entity_type)
				data = self.get_data(entity_type=entity_type, echo=progress_bar-1)
				result.append(data)
				progress += 1
			progress_bar.show(amount=progress)
			self._all_data = concat(result).reset_index(drop=True)
		return self._all_data

	def get_entity_sample(self, size, ratio, data=None, echo=2, random_state=42):
		echo = max(0, echo)
		progress_bar = ProgressBar(total=None, echo=echo)

		if data is None:
			data = self.get_all(echo=progress_bar)

		data = data.sample(frac=1, random_state=random_state)
		counts = data['data_type'].value_counts().to_dict()
		other_size = round(size * ratio)

		dictionary = {}

		progress_bar.set_total(len(counts) * (len(counts) - 1))
		progress_amount = 0
		for main_data_type, count in counts.items():

			result = []
			this_data = data[data['data_type'] == main_data_type]
			if count > size:
				result.append(this_data.head(size))
			else:
				result.append(concat([this_data] * ceil(size / count)).head(size))

			for other_data_type, other_count in counts.items():
				if other_data_type != main_data_type:
					progress_bar.show(amount=progress_amount, text=f'{main_data_type}/{other_data_type}')
					progress_amount += 1
					other_data = data[data['data_type'] == other_data_type]
					if other_count > other_size:
						result.append(other_data.head(other_size))
					else:
						result.append(concat([other_data] * ceil(other_size / other_count)).head(other_size))

			dictionary[main_data_type] = concat(result)

		progress_bar.show(amount=progress_amount, text='sample complete!')

		return dictionary



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

LANDMARK = 'landmark'


def get_landmarks(wikipedia=None, echo=0):
	urls_and_categories = [
		('https://en.wikipedia.org/wiki/List_of_tallest_buildings', ['building', 'tall_building']),
		('https://en.wikipedia.org/wiki/List_of_national_parks_of_the_United_States', ['park', 'national_park', 'united_states']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_eponyms_of_airports'
				'https://en.wikipedia.org/wiki/List_of_busiest_airports_by_passenger_traffic'
			],
			['airport']
		),
		('https://en.wikipedia.org/wiki/List_of_airports_in_China', ['airport', 'china']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_the_United_States', ['port', 'united_states']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Ukraine', ['port', 'ukraine', 'europe']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Sri_Lanka', ['port', 'sri_lanka']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Spain', ['port', 'spain', 'europe']),
		('https://en.wikipedia.org/wiki/Ports_and_harbours_in_South_Africa', ['port', 'south_africa']),
		('https://en.wikipedia.org/wiki/List_of_ports_and_harbours_in_Scotland', ['port', 'scotland', 'europe']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Romania', ['port', 'romania', 'europe']),
		('https://en.wikipedia.org/wiki/Ports_in_India', ['port', 'india']),
		('https://en.wikipedia.org/wiki/List_of_Liverpool_Docks', ['dock', 'liverpool', 'united_kingdom', 'europe']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_China', ['port', 'china']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Belgium', ['port', 'belgium', 'europe']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Australia', ['port', 'australia']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Argentina', ['port', 'argentina']),
		('https://en.wikipedia.org/wiki/List_of_ports_in_Albania', ['port', 'albania', 'europe']),
		('https://en.wikipedia.org/wiki/Iceport', ['port', 'iceport']),
		('https://en.wikipedia.org/wiki/List_of_busiest_ports_in_Europe', ['port', 'busy_port', 'europe']),
		('https://en.wikipedia.org/wiki/List_of_busiest_transshipment_ports', ['port', 'busy_port', 'transshipment_port']),
		('https://en.wikipedia.org/wiki/List_of_busiest_container_ports', ['port', 'busy_port', 'container_port'])
	]

	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)

	return WikipediaLists(
		wikipedia_lists={
			(urls[0] if isinstance(urls, list) else urls): WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=LANDMARK, categories=[LANDMARK] + categories,
				urls=urls,
				columns={LANDMARK: ['port_name', 'name', 'main_ports', 'tourism_ports']}
			)
			for urls, categories in urls_and_categories
		},
		echo=progress_bar
	)



from collections import Counter
from linguistics.english import get_stop_words

_NONPRONOUN_SW = get_stop_words(include_pronoun_forms=False)


def get_signature(
		title_indicator, header_tokens, tokens, info_box, categories, paragraph_statistics,
		summary_tokens, case_sensitive=False
):

	if title_indicator is None:
		title_indicator_list = ['title_indicator:none']
	else:
		title_indicator_list = [f'title_indicator:{title_indicator.lower()}']

	lowercase_header_tokens = [str(header_token).lower() for header_token in header_tokens]
	header_token_list = [f'header_token:{header_token}' for header_token in lowercase_header_tokens]

	lowercase_tokens = [str(token).lower() for token in tokens]
	token_list = [f'token:{token}' for token in lowercase_tokens if token not in _NONPRONOUN_SW]

	lowercase_info_list = [str(info).lower() for info in info_box.keys()]
	info_list = [
		f'info:{info}' for info in lowercase_info_list
		if info not in _NONPRONOUN_SW and not info.startswith('unknown_row_')
	]

	lowercase_category_list = [str(category).lower() for category in categories]
	category_list = [f'category:{category}' for category in lowercase_category_list if category not in _NONPRONOUN_SW]

	summary_list = [f'summary:{token}' for token in summary_tokens if token not in _NONPRONOUN_SW]

	signature = title_indicator_list + header_token_list + token_list + info_list + category_list + summary_list
	if not case_sensitive:
		signature = [x.lower() for x in signature]

	result = dict(Counter(signature))
	result.update(paragraph_statistics)
	return result



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

CONCEPT = 'concept'
LANGUAGE = 'language'


def get_concepts(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	
	return WikipediaLists(
		echo=progress_bar,
		wikipedia_lists={
			'warfare': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'warfare'],
				urls=['https://en.wikipedia.org/wiki/Outline_of_war'],
				representation='list',
				sections=['Types of war']
			),
			'languages': WikipediaListGuide(
				list_type='list_of_lists',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT, categories=[CONCEPT, LANGUAGE],
				urls=[
					'https://en.wikipedia.org/wiki/Lists_of_languages',
					'https://en.wikipedia.org/wiki/Lists_of_endangered_languages'
				],
				exclude_urls=[
					'https://en.wikipedia.org/wiki/List_of_ISO_639-5_codes',
					'https://en.wikipedia.org/wiki/List_of_ISO_639-3_codes',
					'https://en.wikipedia.org/wiki/IETF_language_tag',
					'https://en.wikipedia.org/wiki/Glottolog',
					'https://en.wikipedia.org/wiki/Linguasphere_Observatory',
					'https://en.wikipedia.org/wiki/SIL_International',
					'https://en.wikipedia.org/wiki/International_Organization_for_Standardization',
					'https://en.wikipedia.org/wiki/Index_of_language_articles',
					'https://en.wikipedia.org/wiki/Languages_of_East_Asia',
					'https://en.wikipedia.org/wiki/Languages_of_South_Asia',
					'https://en.wikipedia.org/wiki/List_of_revived_languages',
					'https://en.wikipedia.org/wiki/List_of_lingua_francas'
	
				],
				columns={CONCEPT: [
					'iso_language_name', 'language_name', 'language', 'official_language', 'regional_language',
					'minority_language', 'national_language', 'languages', 'name_of_macrolanguage'
				]}
			),
			'extinct_languages': WikipediaListGuide(
				list_type='list_of_lists',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, LANGUAGE, 'extinct_language', 'extinct'],
				urls=['https://en.wikipedia.org/wiki/Lists_of_extinct_languages'],
				columns={'concept': ['language']}
			),
			'languages_by_country': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT, categories=[CONCEPT, LANGUAGE],
				urls=[
					'https://en.wikipedia.org/wiki/List_of_official_languages_by_country_and_territory',
					'https://en.wikipedia.org/wiki/Afroasiatic_languages',
					'https://en.wikipedia.org/wiki/List_of_major_and_official_Austronesian_languages',
					'https://en.wikipedia.org/wiki/List_of_Indo-European_languages',
					'https://en.wikipedia.org/wiki/List_of_Mongolic_languages',
					'https://en.wikipedia.org/wiki/Tungusic_languages',
					'https://en.wikipedia.org/wiki/List_of_Turkic_languages',
					'https://en.wikipedia.org/wiki/List_of_Uralic_languages',
					'https://en.wikipedia.org/wiki/List_of_languages_by_first_written_accounts',
					'https://en.wikipedia.org/wiki/Languages_of_Afghanistan',
					'https://en.wikipedia.org/wiki/Languages_of_Bangladesh',
					'https://en.wikipedia.org/wiki/Languages_of_India',
					'https://en.wikipedia.org/wiki/Languages_with_official_status_in_India',
					'https://en.wikipedia.org/wiki/List_of_languages_by_number_of_native_speakers_in_India',
					'https://en.wikipedia.org/wiki/Languages_of_Nepal',
					'https://en.wikipedia.org/wiki/Languages_of_Pakistan'
	
				],
				columns={CONCEPT: [
					'language', 'official_language', 'regional_language', 'minority_language', 'national_language',
					'name'
				]}
			),
			'religions': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'religion'],
				urls=['https://en.wikipedia.org/wiki/List_of_religions_and_spiritual_traditions'],
				representation='list'
			),
			'religions_by_founders': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'religion'],
				urls=['https://en.wikipedia.org/wiki/List_of_founders_of_religious_traditions'],
				representation='table', columns={'religion': ['religious_tradition_founded']}
			),
			'shapes': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, categories=[CONCEPT, 'mathematical_concept', 'shape'],
				data_type=CONCEPT,
				urls='https://en.wikipedia.org/wiki/Lists_of_shapes',
				representation='list',
				sections=['mathematics']
			),
			'chemical_elements': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT, categories=[CONCEPT, 'chemical_concept', 'chemical_element'],
				urls='https://en.wikipedia.org/wiki/List_of_chemical_elements',
				columns={CONCEPT: ['element']}
			),
			'colors': WikipediaListGuide(
				list_type='list_of_lists',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT, categories=[CONCEPT, 'colour'],
				urls='https://en.wikipedia.org/wiki/Lists_of_colors',
				exclude_urls=[
					'https://en.wikipedia.org/wiki/List_of_colors_(compact)',
					'https://en.wikipedia.org/wiki/List_of_colors_by_shade',
					'https://en.wikipedia.org/wiki/List_of_color_palettes',
					'https://en.wikipedia.org/wiki/List_of_fictional_colors'
				],
				columns={CONCEPT: ['name']}
			),
			'mathematics_topics': WikipediaListGuide(
				list_type='list_of_lists',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'mathematics_topic'],
				urls='https://en.wikipedia.org/wiki/Lists_of_mathematics_topics',
				representation='list'
			),
			'political_science_topics': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'political_science_topic'],
				urls='https://en.wikipedia.org/wiki/Outline_of_political_science',
				representation='list',
				sections=[
					'Fields of study', 'Political theory', 'Elections', 'Political strategies', 'Political corruption',
					'Government', 'Political philosophies', 'Political issues'
				]
			),
			'time_periods': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'history_topic', 'time_period'],
				urls='https://en.wikipedia.org/wiki/List_of_time_periods',
				representation='list'
			),
			'scientific_laws': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'scientific_law'],
				urls='https://en.wikipedia.org/wiki/List_of_scientific_laws_named_after_people',
				representation='table',
				columns={CONCEPT: ['law']}
			),
			'branches_of_science': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'branch_of_science'],
				urls='https://en.wikipedia.org/wiki/Index_of_branches_of_science',
				representation='list'
			),
			'academic_disciplines': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'academic_discipline'],
				urls=[
					'https://en.wikipedia.org/wiki/Outline_of_academic_disciplines',
					'https://en.wikipedia.org/wiki/List_of_academic_fields'
				],
				representation='list'
			),
			'measurement_units': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'scientific_concept', 'measurement_unit'],
				urls='https://en.wikipedia.org/wiki/List_of_scientific_units_named_after_people',
				representation='list'
			),
			'scientific_constants': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'scientific_concept', 'scientific_constant'],
				urls='https://en.wikipedia.org/wiki/List_of_scientific_constants_named_after_people',
				representation='list'
			),
			'scientific_phenomena': WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=CONCEPT,
				categories=[CONCEPT, 'scientific_concept', 'scientific_phenomenon'],
				urls='https://en.wikipedia.org/wiki/Scientific_phenomena_named_after_people',
				representation='list'
			),
	
		}
	)







from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

GEOGRAPHICAL_FEATURE = 'geographical_feature'


def get_geographical_feature_guides(wikipedia=None, echo=0):
	return {
		'oceans': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[GEOGRAPHICAL_FEATURE, 'body_of_water', 'ocean'],
			data_type=GEOGRAPHICAL_FEATURE,
			urls='https://en.wikipedia.org/wiki/Ocean',
			columns={GEOGRAPHICAL_FEATURE: ['ocean']},
			representation='table'
		),
		'seas': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[GEOGRAPHICAL_FEATURE, 'body_of_water', 'sea'],
			data_type=GEOGRAPHICAL_FEATURE,
			urls='https://en.wikipedia.org/wiki/List_of_seas',
			representation='list'
		),
		'lakes': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, categories=[GEOGRAPHICAL_FEATURE, 'body_of_water', 'lake'],
			data_type=GEOGRAPHICAL_FEATURE,
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_lakes',
				'https://en.wikipedia.org/wiki/Lists_of_lakes_of_Western_Australia'
			],
			columns={GEOGRAPHICAL_FEATURE: ['name', 'lake', 'lake_name', 'lake_or_reservoir']}
		),
		'rivers': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, categories=[GEOGRAPHICAL_FEATURE, 'body_of_water', 'river'],
			data_type=GEOGRAPHICAL_FEATURE,
			urls=[
				'https://en.wikipedia.org/wiki/Lists_of_rivers',
				'https://en.wikipedia.org/wiki/Lists_of_rivers_of_U.S._insular_areas',
				'https://en.wikipedia.org/wiki/List_of_peaks_by_prominence',
				'https://en.wikipedia.org/wiki/Seven_Summits',

			],
			columns={GEOGRAPHICAL_FEATURE: ['river', 'name', 'name_of_river', 'modern_name', 'peak']}
		),
		'mountain_ranges': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[GEOGRAPHICAL_FEATURE, 'mountain_range'],
			data_type=GEOGRAPHICAL_FEATURE,
			urls='https://en.wikipedia.org/wiki/List_of_mountain_ranges',
			representation='list'
		),
		'mountains': WikipediaListGuide(
			list_type='list_of_lists',
			wikipedia=wikipedia, echo=echo - 1, categories=[GEOGRAPHICAL_FEATURE, 'mountain'],
			data_type=GEOGRAPHICAL_FEATURE,
			urls=['https://en.wikipedia.org/wiki/Lists_of_mountains_by_region'],
			columns={GEOGRAPHICAL_FEATURE: ['mountain', 'peak', 'name', 'mountain_peak']}
		),
		'volcanos': WikipediaListGuide(
			list_type='list',
			wikipedia=wikipedia, echo=echo - 1, categories=[GEOGRAPHICAL_FEATURE, 'mountain', 'volcano'],
			data_type=GEOGRAPHICAL_FEATURE,
			urls=[
				'https://en.wikipedia.org/wiki/List_of_large_volcanic_eruptions_of_the_19th_century',
				'https://en.wikipedia.org/wiki/List_of_large_volcanic_eruptions_of_the_20th_century',
				'https://en.wikipedia.org/wiki/List_of_large_volcanic_eruptions_in_the_21st_century',
				'https://en.wikipedia.org/wiki/List_of_volcanic_eruptions_by_death_toll'
			],
			columns={GEOGRAPHICAL_FEATURE: ['volcano', 'volcano_eruption']}
		)
	}

def get_geographical_features(wikipedia=None, echo=0):
	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)
	return WikipediaLists(
		wikipedia_lists=get_geographical_feature_guides(wikipedia=wikipedia, echo=progress_bar),
		echo=progress_bar
	)


from .get_signature import get_signature
from .get_important_tokens import get_important_tokens
from .get_header_tokens import get_header_tokens
from .get_title_indicator import get_title_indicator
from .get_paragraph_statistics import get_paragraph_statistics
from .tokenize import tokenize

from pensieve import Pensieve
from internet.wikipedia import Wikipedia, Page, WIKIPEDIA

class PageSignature:
	def __init__(
			self, page=None, url=None, wikipedia=None, max_num_tokens=100, min_token_length=2,
			evaluate=True, case_sensitive=False
	):
		"""
		:type page: Page or NoneType
		:type url: str or NoneType
		:type wikipedia: Wikipedia or NoneType
		:type max_num_tokens: int
		:type min_token_length: int
		"""
		wikipedia = wikipedia or WIKIPEDIA
		self._pensieve = Pensieve()
		if page is None:
			self.pensieve['url'] = url
			self.pensieve.store(
				key='page', precursors='url', function=lambda x: wikipedia.get_page(url=x), evaluate=evaluate
			)
		else:
			self.pensieve['page'] = page

		self.pensieve['max_num_tokens'] = max_num_tokens
		self.pensieve['min_token_length'] = min_token_length

		# get required attributes
		required_attributes = ['headers', 'title', 'categories', 'info_box', 'summary', 'paragraphs', 'paragraph_links']
		for key in required_attributes:
			self.pensieve.store(key=key, precursors='page', function=lambda x: x[key], evaluate=evaluate)

		self.pensieve.store(key='title_indicator', precursors='title', function=get_title_indicator)
		self.pensieve.store(
			key='paragraph_tokens', precursors='paragraphs', evaluate=evaluate,
			function=lambda x: [tokenize(paragraph.text.lower()) for paragraph in x]
		)

		self.pensieve.store(
			key='paragraph_statistics', precursors=['paragraph_links', 'paragraph_tokens'],
			function=lambda x: get_paragraph_statistics(
				paragraph_links=x['paragraph_links'], paragraph_tokens=x['paragraph_tokens']
			),
			evaluate=evaluate
		)

		self.pensieve.store(
			key='important_tokens', precursors=['paragraph_tokens', 'max_num_tokens', 'min_token_length'],
			function=lambda x: get_important_tokens(
				paragraph_tokens=x['paragraph_tokens'],
				num_tokens=x['max_num_tokens'], min_length=x['min_token_length']),
			evaluate = False
		)

		self.pensieve.store(
			key='header_tokens', precursors=['headers'], evaluate=evaluate,
			function=get_header_tokens
		)

		self.pensieve.store(
			key='summary_tokens', precursors=['summary'], evaluate=evaluate, materialize=False,
			function=tokenize
		)

		self.pensieve.store(
			key='signature',
			precursors=[
				'title_indicator', 'categories', 'info_box', 'important_tokens',
				'header_tokens', 'paragraph_statistics', 'summary_tokens'
			],
			evaluate=evaluate,
			function=lambda x: get_signature(
				title_indicator=x['title_indicator'],
				tokens=x['important_tokens'], info_box=x['info_box'], categories=x['categories'],
				header_tokens=x['header_tokens'], paragraph_statistics=x['paragraph_statistics'],
				summary_tokens=x['summary_tokens'], case_sensitive=case_sensitive
			)
		)

	@property
	def signature(self):
		"""
		:rtype: dict
		"""
		return self.pensieve['signature']

	@property
	def pensieve(self):
		"""
		:rtype: Pensieve
		"""
		return self._pensieve


def get_page_signature(url=None, page=None, wikipedia=None, max_num_tokens=100, min_token_length=2):
	signature = PageSignature(
		page=page, url=url, wikipedia=wikipedia, max_num_tokens=max_num_tokens, min_token_length=min_token_length
	)
	return signature.signature



def get_title_indicator(s):
	open_bracket = s.find('(')
	close_bracket = s.find(')')
	if open_bracket >= 0 and close_bracket >= 0 and open_bracket < close_bracket:
		return s[s.find("(") + 1:s.find(")")]
	else:
		return None



from ..WikipediaListGuide import WikipediaListGuide
from ..WikipediaLists import WikipediaLists
from chronometry.progress import ProgressBar

BAND = 'artist_or_band'
ALBUM = 'album'


def get_music_bands(wikipedia=None, echo=0):
	urls_and_categories = [
		('https://en.wikipedia.org/wiki/List_of_American_grunge_bands', ['pop_rock', 'american_grunge']),
		('https://en.wikipedia.org/wiki/List_of_anarcho-punk_bands', ['pop_rock', 'punk_rock', 'anarchist_punk']),
		('https://en.wikipedia.org/wiki/List_of_music_artists_and_bands_from_Argentina', ['argentine']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_the_San_Francisco_Bay_Area', ['american', 'san_franciscan']),
		('https://en.wikipedia.org/wiki/List_of_Belarusian_musical_groups', ['belarusian']),
		('https://en.wikipedia.org/wiki/List_of_music_artists_and_bands_from_England', ['british']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_British_Columbia', ['canadian', 'british_columbian']),
		('https://en.wikipedia.org/wiki/List_of_British_Invasion_artists', ['pop_rock', 'british_invasion']),
		(
			'https://en.wikipedia.org/wiki/List_of_Chicago_hardcore_punk_bands',
			['pop_rock', 'punk_rock', 'hardcore_punk', 'chicagoan']
		),
		(
			'https://en.wikipedia.org/wiki/List_of_Christian_hardcore_bands',
			['pop_rock', 'punk_rock', 'hardcore_punk', 'christian_hardcore_punk', 'christian_music']
		),
		(
			'https://en.wikipedia.org/wiki/List_of_Christian_rock_bands',
			['pop_rock', 'christian_rock', 'christian_music']
		),
		('https://en.wikipedia.org/wiki/List_of_bands_that_played_at_Dagenham_Roundhouse', []),
		('https://en.wikipedia.org/wiki/List_of_bands_from_Delhi', ['indian', 'delhiite']),
		('https://en.wikipedia.org/wiki/List_of_emo_artists', ['pop_rock', 'emo', 'punk_rock']),
		('https://en.wikipedia.org/wiki/List_of_experimental_big_bands', ['jazz', 'big_band', 'experimental_big_band']),
		('https://en.wikipedia.org/wiki/List_of_extreme_metal_bands', ['pop_rock', 'heavy_metal', 'extreme_metal']),
		('https://en.wikipedia.org/wiki/List_of_all-female_bands', ['all_female_band']),
		('https://en.wikipedia.org/wiki/List_of_girl_groups', ['girl_group']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_Glasgow', ['glaswegians']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_Gothenburg', ['gothenburgian']),
		('https://en.wikipedia.org/wiki/List_of_grunge_bands', ['pop_rock', 'grunge']),
		('https://en.wikipedia.org/wiki/List_of_hardcore_punk_bands', ['pop_rock', 'punk_rock', 'hardcore_punk']),
		('https://en.wikipedia.org/wiki/List_of_instrumental_bands', ['instrumental']),
		('https://en.wikipedia.org/wiki/List_of_jam_bands', ['jam_band']),
		('https://en.wikipedia.org/wiki/List_of_Klezmer_bands', ['klezmer_band']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_Lincoln,_Nebraska', ['american', 'nebraskan']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_Los_Angeles', ['american', 'angeleno']),
		('https://en.wikipedia.org/wiki/List_of_musical_supergroups', ['musical_supergroup']),
		(
			'https://en.wikipedia.org/wiki/List_of_punk_bands_from_the_United_Kingdom',
			['pop_rock', 'punk_rock', 'british']
		),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_punk_rock_bands,_0%E2%80%93K',
				'https://en.wikipedia.org/wiki/List_of_punk_rock_bands,_L%E2%80%93Z',
				'https://en.wikipedia.org/wiki/List_of_1970s_punk_rock_musicians',
				'https://en.wikipedia.org/wiki/List_of_musicians_in_the_second_wave_of_punk_rock'
			],
			['pop_rock', 'punk_rock']
		),
		('https://en.wikipedia.org/wiki/List_of_music_artists_and_bands_from_Manchester', ['british', 'mancunian']),
		('https://en.wikipedia.org/wiki/List_of_bands_and_artists_from_Merseyside', ['british']),
		('https://en.wikipedia.org/wiki/List_of_music_artists_and_bands_from_Mexico', ['mexican']),
		('https://en.wikipedia.org/wiki/List_of_band_name_etymologies', []),
		('https://en.wikipedia.org/wiki/List_of_bands_named_after_other_performers%27_songs', []),
		('https://en.wikipedia.org/wiki/List_of_bands_formed_in_New_York_City', ['american', 'new_yorker']),
		('https://en.wikipedia.org/wiki/List_of_musical_artists_from_New_Zealand', ['kiwi']),
		('https://en.wikipedia.org/wiki/List_of_original_names_of_bands', []),
		('https://en.wikipedia.org/wiki/List_of_Philippine-based_music_groups', ['filipino']),
		('https://en.wikipedia.org/wiki/List_of_pipe_bands', ['pipe_band']),
		('https://en.wikipedia.org/wiki/List_of_post-punk_bands', ['pop_rock', 'punk_rock', 'post_punk']),
		('https://en.wikipedia.org/wiki/List_of_power_trios', ['pop_rock', 'power_trio']),
		('https://en.wikipedia.org/wiki/List_of_riot_grrrl_bands', ['pop_rock', 'punk_rock', 'riot_grrrl']),
		('https://en.wikipedia.org/wiki/List_of_screamo_bands', ['pop_rock', 'punk_rock', 'screamo']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_Spain', ['spanish']),
		('https://en.wikipedia.org/wiki/List_of_straight_edge_bands', ['pop_rock', 'punk_rock', 'straight_edge']),
		('https://en.wikipedia.org/wiki/List_of_bands_from_Taiwan', ['taiwanese']),
		('https://en.wikipedia.org/wiki/List_of_visual_kei_musical_groups', ['visual_kei_musical_group']),
		('https://en.wikipedia.org/wiki/List_of_progressive_rock_artists', ['pop_rock', 'progressive_rock']),
		('https://en.wikipedia.org/wiki/List_of_heavy_metal_bands', ['pop_rock', 'heavy_metal']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_black_metal_bands,_0%E2%80%93K',
				'https://en.wikipedia.org/wiki/List_of_black_metal_bands,_L%E2%80%93Z'
			],
			['pop_rock', 'black_metal']
		),
		('https://en.wikipedia.org/wiki/List_of_acid_rock_artists', ['pop_rock', 'acid_rock']),
		('https://en.wikipedia.org/wiki/List_of_new-age_music_artists', ['new_age']),
		('https://en.wikipedia.org/wiki/List_of_thrash_metal_bands', ['pop_rock', 'heavy_metal', 'thrash_metal']),
		('https://en.wikipedia.org/wiki/List_of_speed_metal_bands', ['pop_rock', 'heavy_metal', 'speed_metal']),
		('https://en.wikipedia.org/wiki/List_of_industrial_metal_bands', ['pop_rock', 'industrial_metal']),
		('https://en.wikipedia.org/wiki/List_of_alternative_rock_artists', ['pop_rock', 'alternative_pop_rock']),
		('https://en.wikipedia.org/wiki/List_of_adult_alternative_artists', ['pop_rock', 'adult_alternative_pop_rock']),
		('https://en.wikipedia.org/wiki/List_of_soft_rock_artists_and_songs', ['pop_rock', 'soft_rock']),
		('https://en.wikipedia.org/wiki/Adult_Contemporary_(chart)', ['pop_rock', 'soft_rock', 'adult_contemporary']),
		('https://en.wikipedia.org/wiki/List_of_dream_pop_artists', ['pop_rock', 'alternative_pop_rock', 'dream_pop']),
		('https://en.wikipedia.org/wiki/List_of_synth-pop_artists', ['pop_rock', 'synth_pop']),
		('https://en.wikipedia.org/wiki/List_of_southern_rock_bands', ['pop_rock', 'southern_rock']),
		('https://en.wikipedia.org/wiki/List_of_folk_rock_artists', ['pop_rock', 'folk_rock']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_country_performers_by_era',
				'https://en.wikipedia.org/wiki/List_of_artists_who_reached_number_one_on_the_U.S._country_chart'
			],
			['country']
		),
		(
			'https://en.wikipedia.org/wiki/List_of_Christian_country_artists',
			['country', 'christian_country', 'christian_music']
		),
		('https://en.wikipedia.org/wiki/List_of_blues_rock_musicians', ['pop_rock', 'blues_rock']),
		('https://en.wikipedia.org/wiki/List_of_blues_musicians', ['blues']),
		('https://en.wikipedia.org/wiki/List_of_best-selling_music_artists', ['best_selling_artist']),
		('https://en.wikipedia.org/wiki/List_of_Eurobeat_artists', ['eurobeat']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(A%E2%80%93E)',
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(F%E2%80%93K)',
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(L%E2%80%93R)',
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(S%E2%80%93Z)'
			],
			['disco']
		),
		('https://en.wikipedia.org/wiki/List_of_dance-pop_artists', ['dance_pop']),
		('https://en.wikipedia.org/wiki/List_of_alternative_hip_hop_artists', ['hip_hop', 'alternative_hip_hop']),
		('https://en.wikipedia.org/wiki/List_of_Christian_hip_hop_artists', ['hip_hop', 'christian_hip_hop']),
		('https://en.wikipedia.org/wiki/List_of_gothic_rock_artists', ['pop_rock', 'gothic_rock']),
		('https://en.wikipedia.org/wiki/List_of_electronic_rock_artists', ['pop_rock', 'electronic_rock']),
		('https://en.wikipedia.org/wiki/List_of_1970s_Christian_pop_artists', ['christian_pop']),
		('https://en.wikipedia.org/wiki/List_of_alternative_country_musicians', ['country', 'alternative_country']),
		(
			'https://en.wikipedia.org/wiki/List_of_alternative_metal_artists',
			['pop_rock', 'heavy_metal', 'alternative_metal']
		),
		('https://en.wikipedia.org/wiki/List_of_baroque_pop_artists', ['baroque_pop']),
		('https://en.wikipedia.org/wiki/List_of_bebop_musicians', ['bepop']),
		('https://en.wikipedia.org/wiki/List_of_bhangra_artists', ['bhangra']),
		('https://en.wikipedia.org/wiki/List_of_big_band_musicians', ['big_band']),
		('https://en.wikipedia.org/wiki/List_of_blue-eyed_soul_artists', ['soul', 'blue_eyed_soul']),
		('https://en.wikipedia.org/wiki/List_of_bluegrass_musicians', ['blue_grass']),
		('https://en.wikipedia.org/wiki/List_of_C-pop_artists', ['pop_rock', 'c_pop']),
		('https://en.wikipedia.org/wiki/List_of_calypso_musicians', ['calypso']),
		('https://en.wikipedia.org/wiki/List_of_Celtic_musicians', ['celtic']),
		('https://en.wikipedia.org/wiki/List_of_chamber_jazz_musicians', ['jazz', 'chamber_jazz']),
		('https://en.wikipedia.org/wiki/List_of_Chicago_blues_musicians', ['blues', 'chicago_blues']),
		(
			'https://en.wikipedia.org/wiki/List_of_Christian_dance,_electronic,_and_techno_artists',
			['christian_music', 'christian_electronic']
		),
		(
			'https://en.wikipedia.org/wiki/List_of_Christian_metal_artists',
			['pop_rock', 'heavy_metal', 'christian_metal', 'christian_music']
		),
		('https://en.wikipedia.org/wiki/List_of_dance-punk_artists', ['dance_punk']),
		('https://en.wikipedia.org/wiki/List_of_dance-rock_artists', ['pop_rock', 'dance_rock']),
		('https://en.wikipedia.org/wiki/List_of_dark_ambient_artists', ['dark_ambient']),
		('https://en.wikipedia.org/wiki/List_of_dark_cabaret_artists', ['dark_cabaret']),
		('https://en.wikipedia.org/wiki/List_of_dark_rock_bands', ['pop_rock', 'dark_rock']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_death_metal_bands,_!%E2%80%93K',
				'https://en.wikipedia.org/wiki/List_of_death_metal_bands,_L%E2%80%93Z'
			],
			['pop_rock', 'heavy_metal', 'death_metal']
		),
		('https://en.wikipedia.org/wiki/List_of_deathcore_artists', ['pop_rock', 'heavy_metal', 'deathcore']),
		('https://en.wikipedia.org/wiki/List_of_Delta_blues_musicians', ['blues', 'delta_blues']),
		('https://en.wikipedia.org/wiki/List_of_electric_blues_musicians', ['blues', 'electric_blues']),
		('https://en.wikipedia.org/wiki/List_of_electro_house_artists', ['electro_house']),
		('https://en.wikipedia.org/wiki/List_of_electro-industrial_bands', ['electro_industrial']),
		('https://en.wikipedia.org/wiki/List_of_electroclash_bands_and_artists', ['electroclash']),
		('https://en.wikipedia.org/wiki/List_of_fado_musicians', ['fado']),
		('https://en.wikipedia.org/wiki/List_of_folk_musicians', ['folk']),
		('https://en.wikipedia.org/wiki/List_of_folk_metal_bands', ['pop_rock', 'heavy_metal', 'folk_metal']),
		('https://en.wikipedia.org/wiki/List_of_G-funk_musicians', ['g_funk']),
		('https://en.wikipedia.org/wiki/List_of_gangsta_rap_artists', ['gangsta_rap']),
		('https://en.wikipedia.org/wiki/List_of_garage_rock_bands', ['pop_rock', 'garage_rock']),
		(
			'https://en.wikipedia.org/wiki/List_of_glam_metal_bands_and_artists',
			['pop_rock', 'heavy_metal', 'glam_metal']
		),
		('https://en.wikipedia.org/wiki/List_of_glam_punk_artists', ['pop_rock', 'punk_rock', 'glam_punk']),
		('https://en.wikipedia.org/wiki/List_of_glam_rock_artists', ['pop_rock', 'glam_rock']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_hard_rock_musicians_(A%E2%80%93M)',
				'https://en.wikipedia.org/wiki/List_of_hard_rock_musicians_(N%E2%80%93Z)'
			],
			['pop_rock', 'hard_rock']
		),
		('https://en.wikipedia.org/wiki/List_of_indie_pop_artists', ['indie_pop']),
		('https://en.wikipedia.org/wiki/List_of_indie_rock_musicians', ['pop_rock', 'indie_rock']),
		('https://en.wikipedia.org/wiki/List_of_Indonesian_pop_musicians', ['indonesian_pop']),
		('https://en.wikipedia.org/wiki/List_of_J-pop_artists', ['j_pop']),
		('https://en.wikipedia.org/wiki/List_of_Japanoise_artists', ['japanoise']),
		('https://en.wikipedia.org/wiki/List_of_jazz_fusion_musicians', ['jazz', 'jazz_fusion']),
		('https://en.wikipedia.org/wiki/List_of_K-pop_artists', ['k_pop']),
		('https://en.wikipedia.org/wiki/List_of_Latin_American_rock_musicians', ['pop_rock', 'latin_american', 'rock']),
		('https://en.wikipedia.org/wiki/List_of_Latin_pop_artists', ['latin_pop']),
		('https://en.wikipedia.org/wiki/List_of_maritime_music_performers', ['maritime_music']),
		('https://en.wikipedia.org/wiki/List_of_math_rock_groups', ['pop_rock', 'math_rock']),
		('https://en.wikipedia.org/wiki/List_of_mathcore_bands', ['mathcore']),
		('https://en.wikipedia.org/wiki/List_of_merengue_musicians', ['merengue_music']),
		('https://en.wikipedia.org/wiki/List_of_new_jack_swing_artists', ['new_jack_swing']),
		('https://en.wikipedia.org/wiki/List_of_new_wave_artists', ['new_wave']),
		('https://en.wikipedia.org/wiki/List_of_Oi!_bands', ['pop_rock', 'punk_rock', 'oi_band']),
		('https://en.wikipedia.org/wiki/List_of_Piedmont_blues_musicians', ['blues', 'piedmont_blues']),
		('https://en.wikipedia.org/wiki/List_of_political_hip_hop_artists', ['hip_hop', 'political_hip_hop']),
		('https://en.wikipedia.org/wiki/List_of_polka_artists', ['polka']),
		('https://en.wikipedia.org/wiki/List_of_pop_punk_bands', ['pop_punk']),
		('https://en.wikipedia.org/wiki/List_of_post-disco_artists_and_songs', ['post_disco']),
		('https://en.wikipedia.org/wiki/List_of_post-dubstep_musicians', ['post_dubstep']),
		('https://en.wikipedia.org/wiki/List_of_post-grunge_bands', ['pop_rock', 'post_grunge']),
		('https://en.wikipedia.org/wiki/List_of_post-hardcore_bands', ['pop_rock', 'punk_rock', 'post_hardcore']),
		('https://en.wikipedia.org/wiki/Post-metal', ['pop_rock', 'heavy_metal', 'post_metal']),
		('https://en.wikipedia.org/wiki/List_of_post-punk_revival_bands', ['pop_rock', 'punk_rock', 'post_punk_revival']),
		('https://en.wikipedia.org/wiki/List_of_post-rock_bands', ['pop_rock', 'post_rock']),
		('https://en.wikipedia.org/wiki/List_of_power_metal_bands', ['pop_rock', 'heavy_metal', 'power_metal']),
		('https://en.wikipedia.org/wiki/List_of_power_pop_artists_and_songs', ['power_pop']),
		('https://en.wikipedia.org/wiki/List_of_R%26B_musicians', ['rhythm_and_blues']),
		('https://en.wikipedia.org/wiki/List_of_ragtime_musicians', ['ragtime']),
		('https://en.wikipedia.org/wiki/List_of_ra%C3%AF_musicians', ['rai']),
		('https://en.wikipedia.org/wiki/List_of_rap_rock_bands', ['rap_rock']),
		('https://en.wikipedia.org/wiki/List_of_reggae_musicians', ['reggae']),
		('https://en.wikipedia.org/wiki/List_of_reggae_fusion_artists', ['reggae', 'reggae_fusion']),
		('https://en.wikipedia.org/wiki/List_of_reggae_rock_artists', ['reggae', 'reggae_rock']),
		('https://en.wikipedia.org/wiki/List_of_shoegazing_musicians', ['pop_rock', 'alternative_pop_rock', 'shoegazing']),
		('https://en.wikipedia.org/wiki/List_of_ska_musicians', ['ska']),
		('https://en.wikipedia.org/wiki/List_of_smooth_jazz_musicians', ['jazz', 'smooth_jazz']),
		('https://en.wikipedia.org/wiki/List_of_soul_musicians', ['soul']),
		('https://en.wikipedia.org/wiki/List_of_soul-blues_musicians', ['blues', 'soul_blues']),
		('https://en.wikipedia.org/wiki/List_of_technical_death_metal_bands', ['pop_rock', 'heavy_metal', 'death_metal', 'technical_death_metal']),
		('https://en.wikipedia.org/wiki/List_of_Texas_blues_musicians', ['blues', 'texas_blues']),
		('https://en.wikipedia.org/wiki/List_of_Thai_pop_artists', ['thai_pop']),
		('https://en.wikipedia.org/wiki/List_of_UK_garage_artists', ['electronic_music', 'uk_garage']),
		('https://en.wikipedia.org/wiki/List_of_Viking_metal_bands', ['pop_rock', 'heavy_metal', 'viking_metal']),
		('https://en.wikipedia.org/wiki/List_of_West_Coast_blues_musicians', ['blues', 'west_coast_blues']),
		('https://en.wikipedia.org/wiki/List_of_ambient_music_artists', ['ambient_music']),
		('https://en.wikipedia.org/wiki/List_of_Arabic_pop_musicians', ['arabic_pop']),
		('https://en.wikipedia.org/wiki/List_of_boogie_woogie_musicians', ['boogie_woogie']),
		('https://en.wikipedia.org/wiki/List_of_British_blues_musicians', ['blues', 'british_blues']),
		('https://en.wikipedia.org/wiki/List_of_Britpop_musicians', ['pop_rock', 'britpop']),
		('https://en.wikipedia.org/wiki/List_of_Christian_ska_bands', ['pop_rock', 'ska', 'christian_ska', 'christian_music']),
		('https://en.wikipedia.org/wiki/List_of_classic_female_blues_singers', ['blues', 'classic_female_blues']),
		('https://en.wikipedia.org/wiki/List_of_cool_jazz_and_West_Coast_jazz_musicians', ['jazz', 'cool_jazz']),
		('https://en.wikipedia.org/wiki/List_of_country_blues_musicians', ['blues', 'country_blues']),
		('https://en.wikipedia.org/wiki/List_of_country_music_performers', ['country']),
		('https://en.wikipedia.org/wiki/List_of_country_rock_musicians', ['country_rock']),
		('https://en.wikipedia.org/wiki/List_of_crooners', ['crooner']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(A%E2%80%93E)',
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(F%E2%80%93K)',
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(L%E2%80%93R)',
				'https://en.wikipedia.org/wiki/List_of_disco_artists_(S%E2%80%93Z)'
			],
			['disco']
		),
		('https://en.wikipedia.org/wiki/List_of_doo-wop_musicians', ['doo_wop']),
		('https://en.wikipedia.org/wiki/List_of_doom_metal_bands', ['pop_rock', 'heavy_metal', 'doom_metal']),
		('https://en.wikipedia.org/wiki/List_of_downtempo_artists', ['electronic_music', 'downtempo']),
		('https://en.wikipedia.org/wiki/List_of_drone_artists', ['drone_music']),
		('https://en.wikipedia.org/wiki/List_of_dub_artists', ['reggae', 'dub_music', 'electronic_music']),
		('https://en.wikipedia.org/wiki/List_of_dubstep_musicians', ['electronic_music', 'dance_music', 'dubstep']),
		('https://en.wikipedia.org/wiki/List_of_Euro_disco_artists', ['dance_music', 'euro_disco']),
		('https://en.wikipedia.org/wiki/List_of_Eurobeat_artists', ['dance_music', 'eurobeat']),
		('https://en.wikipedia.org/wiki/List_of_Eurodance_artists', ['dance_music', 'eurodance']),
		('https://en.wikipedia.org/wiki/List_of_Europop_artists', ['europop']),
		('https://en.wikipedia.org/wiki/List_of_free_funk_musicians', ['jazz', 'fusion', 'free_funk']),
		('https://en.wikipedia.org/wiki/List_of_funk_musicians', ['rhythm_and_blues', 'soul', 'funk']),
		('https://en.wikipedia.org/wiki/List_of_funk_rock_bands', ['pop_rock', 'funk_rock']),
		('https://en.wikipedia.org/wiki/List_of_gospel_blues_musicians', ['blues', 'gospel_blues']),
		('https://en.wikipedia.org/wiki/List_of_gospel_musicians', ['gospel']),
		('https://en.wikipedia.org/wiki/List_of_gothic_metal_bands', ['pop_rock', 'heavy_metal', 'gothic_metal']),
		('https://en.wikipedia.org/wiki/List_of_grindcore_bands', ['pop_rock', 'heavy_metal', 'punk_rock', 'grindcore']),
		('https://en.wikipedia.org/wiki/List_of_groove_metal_bands', ['pop_rock', 'heavy_metal', 'groove_metal']),
		(
			[
				'https://en.wikipedia.org/wiki/List_of_hip_hop_groups',
				'https://en.wikipedia.org/wiki/List_of_hip_hop_musicians'
			],
			['hip_hop']
		),
		('https://en.wikipedia.org/wiki/List_of_horror_punk_bands', ['pop_rock', 'punk_rock', 'horror_punk']),
		('https://en.wikipedia.org/wiki/List_of_house_music_artists', ['dance_music', 'house_music']),
		('https://en.wikipedia.org/wiki/List_of_industrial_music_bands', ['industrial_music']),
		('https://en.wikipedia.org/wiki/List_of_intelligent_dance_music_artists', ['electronic_music', 'intelligent_dance']),
		('https://en.wikipedia.org/wiki/List_of_Italo_disco_artists_and_songs', ['disco', 'italo_disco']),
		('https://en.wikipedia.org/wiki/List_of_jazz_musicians', ['jazz']),
		('https://en.wikipedia.org/wiki/List_of_jump_blues_musicians', ['blues', 'jump_blues']),
		('https://en.wikipedia.org/wiki/List_of_jungle_and_drum_and_bass_artists', ['jungle_and_drum_and_bass', 'electronic_music']),
		('https://en.wikipedia.org/wiki/List_of_klezmer_musicians', ['klezmer']),
		('https://en.wikipedia.org/wiki/List_of_lovers_rock_artists', ['reggae', 'lovers_rock']),
		('https://en.wikipedia.org/wiki/List_of_metalcore_bands', ['pop_rock', 'heavy_metal', 'metalcore']),
		('https://en.wikipedia.org/wiki/List_of_melodic_death_metal_bands', ['pop_rock', 'heavy_metal', 'melodic_death_metal']),
		('https://en.wikipedia.org/wiki/List_of_new_wave_of_British_heavy_metal_bands', ['pop_rock', 'heavy_metal', 'new_wave_of_british_metal']),
		('https://en.wikipedia.org/wiki/List_of_noise_musicians', ['noise_music']),
		('https://en.wikipedia.org/wiki/List_of_nu_metal_bands', ['pop_rock', 'heavy_metal', 'alternative_metal', 'nu_metal']),
		('https://en.wikipedia.org/wiki/List_of_progressive_house_artists', ['house_music', 'progressive_house']),
		('https://en.wikipedia.org/wiki/List_of_progressive_metal_artists', ['pop_rock', 'heavy_metal', 'progressive_metal']),
		('https://en.wikipedia.org/wiki/List_of_progressive_rock_supergroups', ['progressive_rock', 'supergroup']),
		('https://en.wikipedia.org/wiki/List_of_psychedelic_folk_artists', ['folk', 'psychedelic', 'psychedelic_folk']),
		('https://en.wikipedia.org/wiki/List_of_psychedelic_pop_artists', ['psychedelic', 'psychedelic_pop']),
		('https://en.wikipedia.org/wiki/List_of_psychedelic_rock_artists', ['psychedelic', 'psychedelic_rock']),
		('https://en.wikipedia.org/wiki/List_of_psychobilly_bands', ['pop_rock', 'punk_rock', 'psychobilly']),
		('https://en.wikipedia.org/wiki/List_of_punk_blues_musicians_and_bands', ['pop_rock', 'punk_rock', 'punk_blues']),
		('https://en.wikipedia.org/wiki/List_of_reggaeton_musicians', ['reggaeton']),
		('https://en.wikipedia.org/wiki/List_of_rock_music_performers', ['pop_rock', 'rock']),
		('https://en.wikipedia.org/wiki/List_of_rocksteady_musicians', ['rocksteady']),
		('https://en.wikipedia.org/wiki/List_of_roots_reggae_artists', ['reggae', 'roots_reggae']),
		('https://en.wikipedia.org/wiki/List_of_roots_rock_bands_and_musicians', ['pop_rock', 'roots_rock']),
		('https://en.wikipedia.org/wiki/List_of_soul_jazz_musicians', ['jazz', 'soul_jazz']),
		('https://en.wikipedia.org/wiki/List_of_street_punk_bands', ['punk_rock', 'street_punk']),
		('https://en.wikipedia.org/wiki/List_of_surf_musicians', ['pop_rock', 'surf_music']),
		('https://en.wikipedia.org/wiki/List_of_swing_musicians', ['swing']),
		('https://en.wikipedia.org/wiki/List_of_symphonic_metal_bands', ['pop_rock', 'heavy_metal', 'symphonic_metal']),
		('https://en.wikipedia.org/wiki/Thrashcore', ['pop_rock', 'punk_rock', 'thrashcore']),
		('https://en.wikipedia.org/wiki/List_of_trip_hop_artists', ['trip_hop']),
		('https://en.wikipedia.org/wiki/List_of_vocal_groups', ['vocal_group']),
		('https://en.wikipedia.org/wiki/List_of_vocal_trance_artists', ['electronic_music', 'trance', 'vocal_trance']),
		('https://en.wikipedia.org/wiki/List_of_West_Coast_hip_hop_artists', ['hip_hop', 'west_coast'])

	]

	echo = max(0, echo)
	progress_bar = ProgressBar(echo=echo, total=None)

	return WikipediaLists(
		wikipedia_lists={
			(urls[0] if isinstance(urls, list) else urls): WikipediaListGuide(
				list_type='list',
				wikipedia=wikipedia, echo=progress_bar - 1, data_type=BAND, categories=[BAND] + categories, urls=urls,
				columns={
					BAND: ['band', 'band_project_name', 'band_name', 'name', 'artist', 'project_name', 'artist_name']
				}
			)
			for urls, categories in urls_and_categories
		},
		echo=progress_bar
	)



from .WikipediaList import WikipediaList


class WikipediaListGuide:
	def __init__(
			self, list_type, data_type, urls, categories, wikipedia,
			echo, representation=None,
			use_cache_if_available=True, exclude_urls=None, columns=None, sections=None,
			exclude_pages=None, exclude_list_pages=None
	):
		"""
		:type list_type: str
		:type data_type: str
		:type urls: str or list[str]
		:type categories: list[str]
		:type wikipedia: Wikipedia
		:type echo: NoneType or int or ProgressBar
		:type representation: str or NoneType
		:type use_cache_if_available: bool
		:type exclude_urls: NoneType or list[str]
		:type columns: dict[str, list] or dict[str, str]
		:type sections: list[str]
		:type exclude_pages: NoneType or list[str]
		:type exclude_list_pages: NoneType or list[str]
		"""
		self.list_type = list_type
		self.data_type = data_type
		self.urls = urls
		self.categories = categories
		self.wikipedia = wikipedia
		self.columns = columns
		self.representation = representation
		self.sections = sections
		self.exclude_urls = exclude_urls
		self.exclude_list_pages = exclude_list_pages
		self.exclude_pages = exclude_pages
		self.use_cache_if_available = use_cache_if_available
		self.echo = echo

	def get_list(self):
		if self.list_type == 'list_of_lists' or self.list_type == 'list_of_list':
			return WikipediaList.of_lists(
				data_type=self.data_type, urls=self.urls, categories=self.categories,
				wikipedia=self.wikipedia, columns=self.columns, representation=self.representation,
				sections=self.sections, exclude_urls=self.exclude_urls, echo=self.echo
			)
		elif self.list_type == 'list':
			return WikipediaList(
				urls=self.urls, wikipedia=self.wikipedia, categories=self.categories,
				representation=self.representation, data_type=self.data_type, sections=self.sections,
				columns=self.columns, exclude_list_pages=self.exclude_list_pages, exclude_pages=self.exclude_pages,
				use_cache_if_available=self.use_cache_if_available, echo=self.echo
			)


def get_important_tokens(paragraph_tokens, num_tokens, min_length):
	"""
	:param paragraphs soup: list[Tag]
	:param int num_tokens: number of tokens required to finish the job
	:param int min_length: minimum length for a token to be considered (anything shorter will be ignored)
	:rtype: list[str]
	"""
	important_tokens = []
	for tokens in paragraph_tokens:
		if len(important_tokens) >= num_tokens:
			break
		important_tokens += [token for token in tokens if len(token) >= min_length]
	return important_tokens
